
/* These are C/Pascal definitions of various driver routines for SP/TH's consumption */

	/* prodos2.aii */
#define READ_ENABLE 1
#define WRITE_ENABLE 2
pascal unsigned D_OPEN2(/* PathNamePtr:long, Access:word */);

pascal void D_CLOSE2(/* RefId:word */);

pascal void D_CREATE2(/* PathNamePtr:long, Type:word, AuxType:long */);

pascal void D_DESTROY2(/* PathNamePtr:long */);

pascal unsigned long D_READ2(/* RefId:word, BufferPtr:long, RequestCount:long */);

pascal unsigned long D_WRITE2(/* RefId:word, BufferPtr:long, RequestCount:long */);

pascal unsigned long D_GETMARK2(/* RefId:word */);

pascal void D_SETMARK2(/* RefId:word, Base:word, Displacement:long */);

pascal unsigned long D_GETEOF2(/* RefId:word */);

pascal void D_SETEOF2(/* RefId:word, Base:word, Displacement:long */);

	/* memutil.aii */
pascal void D_DISPOSEPTR(/* ptr:long */);

pascal char **D_MUSTHANDLE(/* size:long */);

pascal char **D_NEEDHANDLEID(/* size:long,attr:word,id:w */);

pascal void D_SETCURSOR(/* 1=Watch, 0=Arrow */);
#define arrowCursor 0
#define watchCursor 1

pascal unsigned D_ALERTBOX(/* type:w, str:l */);
#define okBox		1
#define okCancelBox	2
#define yesNoBox	3
#define yesNoNoCancelBox	4

pascal void D_BEACHBALL();
pascal void D_BLINKBUTTON(/* dialog:l, item:w */);